from ClientsRepository import *
repo = ClientsRepository("mongodb+srv://doadmin:g12k79jU8L3y0u5t@db-mongodb-nyc3-12601-daeda50b.mongo.ondigitalocean.com/admin?authSource=admin&replicaSet=db-mongodb-nyc3-12601&tls=true")
repo.add_client("f4f35d119d7cd5df3f5a327c92ccc107")