import setuptools

setuptools.setup(
    name="Federated-Learning-Server",
    version="0.0.8",
    author="Bichebi",
    author_email="mgach18@freeuni.edu.ge",
    description="Server package for Collaborative learning framework for deep neural network",
    packages=["Server"],
    url="https://github.com/sHOTa-23/Collaborative-Learning-Framework-for-Deep-Neural-Network"
)
